@@ -15,8 +15,8 @@ from mypvdevices.ChannelConfig import ChannelConfig
 from mypvdevices.SetupMapping import SetupMapping
 
 SERIALLEN = 16
-ACTEMPMINIMUMVALUE = 40
-ACTEMPMAXIMUMVALUE = 90
+ACTEMPMINIMUMVALUE = 5
+ACTEMPMAXIMUMVALUE = 99
 
 class ElwaSc20(SerialDevice):
 
@@ -83,8 +83,10 @@ class ElwaSc20(SerialDevice):
         self._channels["internal_control_value_4"] = ChannelConfig([28], "NUMBER", 0, registertype="serial")
         # self._channels["8bit_checksum"] = ChannelConfig([29], "NUMBER", 0, registertype="serial")
 
+        self._setup["fwversion"] = SetupMapping(defaultvalue=self._firmwareversion) #required to find write configuration in cloud
         self._setup["device"] = SetupMapping(defaultvalue=self._devicesubtype)
-        self._setup["fwversion"] = SetupMapping(defaultvalue=self._firmwareversion)
+        self._setup["firmware"] = SetupMapping("fw_version")
+        self._setup["libversion"] = SetupMapping(defaultvalue=self._firmwareversion)
         self._setup["serialno"] = SetupMapping(defaultvalue=self._serial)
         self._setup["ww1boost"] = SetupMapping("ac_temp_target")
 
@@ -122,7 +124,7 @@ class ElwaSc20(SerialDevice):
 
     def getdata(self):
         acRelayState = self.getchannelvalue("ac_relay_state", True)
-        dcPower = self.getchannelvalue("dc_power", True)
+        dcPower = self.getchannelvalue("dc_power", True, 10)
         operationMode = self.getchannelvalue("operation_mode", True)
 
         if operationMode == 3 or operationMode == 5 or operationMode == 6 or operationMode == 7 or operationMode == 8 or operationMode == 9 or operationMode == 10 or operationMode == 11 or operationMode == 12 or operationMode == 13 or operationMode == 14 or operationMode == 15 or operationMode == 16 or operationMode == 20 or operationMode == 21 or operationMode == 135:
@@ -167,8 +169,8 @@ class ElwaSc20(SerialDevice):
             "ac_temp_target": self.getchannelvalue("ac_temp_target", True, 10),
             "tempchip": self.getchannelvalue("tempchip", True),
             "iso_voltage": self.getchannelvalue("iso_voltage", True),
-            "dc_voltage": self.getchannelvalue("dc_voltage", True),
-            "dc_current": self.getchannelvalue("dc_current", True),
+            "dc_voltage": self.getchannelvalue("dc_voltage", True, 10),
+            "dc_current": self.getchannelvalue("dc_current", True, 10),
             "boostpower": boostpower,
             "dc_day_wh": self.getchannelvalue("dc_day_wh", True),
             "dc_total_kwh": self.getchannelvalue("dc_total_kwh", True),
@@ -260,19 +262,21 @@ if __name__ == "__main__":
     # device.readallregisters()
     # print(device)
 
-    device = ElwaSc20(serialnr)
+    # device = ElwaSc20(serialnr)
     # setup = device.getsetup()
     # print(setup)
-    # device.setsetupvalue("ww2target", 55)
+    # device.setsetupvalue("ww1boost", 43)
     # setup = device.getsetup()
     # print(setup)
     # device.setsetupvalue("test", 55)
     # setup = device.getsetup()
     # print(setup)
 
-    device.readallregisters()
-    device._syncsettings()
-    print(device.getsetup())
+    # device = ElwaSc20(serialnr)
+    # device.readallregisters()
+    # device._syncsettings()
+    # device._syncsettings()
+    # print(device.getsetup())
     
     # register = device.readregisters(0, 30, "serial")
     # print(register)
@@ -329,93 +333,94 @@ if __name__ == "__main__":
 
     # print(device)
 
-    #AUTO-Tests
-    try:
-        device = ElwaSc20("test")
-        print(color('ERROR: creating invalid device.', fore='red', style='bright'))
-    except:
-        print(color('SUCCESS: creating invalid device.', fore='green', style='bright'))
+    # #AUTO-Tests
+    # try:
+    #     device = ElwaSc20("test")
+    #     print(color('ERROR: creating invalid device.', fore='red', style='bright'))
+    # except:
+    #     print(color('SUCCESS: creating invalid device.', fore='green', style='bright'))
 
-    try:
-        device = ElwaSc20("test")
-        print(color('ERROR: creating invalid device - none.', fore='red', style='bright'))
-    except:
-        print(color('SUCCESS: creating invalid device - none.', fore='green', style='bright'))
+    # try:
+    #     device = ElwaSc20("test")
+    #     print(color('ERROR: creating invalid device - none.', fore='red', style='bright'))
+    # except:
+    #     print(color('SUCCESS: creating invalid device - none.', fore='green', style='bright'))
 
-    try:
-        device = ElwaSc20(serialnr)
-        print(color('SUCCESS: creating valid device.', fore='green', style='bright'))
-    except:
-        print(color('ERROR: creating valid device.', fore='red', style='bright'))
+    # try:
+    #     device = ElwaSc20(serialnr)
+    #     print(color('SUCCESS: creating valid device.', fore='green', style='bright'))
+    # except:
+    #     print(color('ERROR: creating valid device.', fore='red', style='bright'))
 
-    try:
-        device._connect()
-        print(color('SUCCESS: connecting device.', fore='green', style='bright'))
-    except Exception as e:
-        print(color('ERROR: connecting device.', fore='red', style='bright'))
+    # try:
+    #     device._connect()
+    #     print(color('SUCCESS: connecting device.', fore='green', style='bright'))
+    # except Exception as e:
+    #     print(color('ERROR: connecting device.', fore='red', style='bright'))
 
-    try:
-        device.readallregisters()
-        print(color('SUCCESS: reading registers.', fore='green', style='bright'))
-    except:
-        print(color('ERROR: reading registers.', fore='red', style='bright'))
+    # try:
+    #     device.readallregisters()
+    #     print(color('SUCCESS: reading registers.', fore='green', style='bright'))
+    # except:
+    #     print(color('ERROR: reading registers.', fore='red', style='bright'))
 
-    try:
-        setup = device.getsetup()
-        if(setup == None):
-            raise Exception("Setup is None")
-        print(color('SUCCESS: getting device Setup.', fore='green', style='bright'))
-    except Exception as e:
-        print(color('ERROR: getting device Setup. Message: ' + str(e), fore='red', style='bright'))
+    # try:
+    #     setup = device.getsetup()
+    #     if(setup == None):
+    #         raise Exception("Setup is None")
+    #     print(color('SUCCESS: getting device Setup.', fore='green', style='bright'))
+    # except Exception as e:
+    #     print(color('ERROR: getting device Setup. Message: ' + str(e), fore='red', style='bright'))
 
-    try:
-        data = device.getdata()
-        if(data == None):
-            raise Exception("Data is None")
-        print(color('SUCCESS: getting device data.', fore='green', style='bright'))
-    except Exception as e:
-        print(color('ERROR: getting device data.', fore='red', style='bright'))
+    # try:
+    #     data = device.getdata()
+    #     if(data == None):
+    #         raise Exception("Data is None")
+    #     print(color('SUCCESS: getting device data.', fore='green', style='bright'))
+    # except Exception as e:
+    #     print(color('ERROR: getting device data.', fore='red', style='bright'))
 
-    try:
-        logdata = device.getlogdata()
-        if(logdata == None):
-            raise Exception("logdata is None")
-        print(color('SUCCESS: getting device logdata.', fore='green', style='bright'))
-    except:
-        print(color('ERROR: getting device logdata.', fore='red', style='bright'))
+    # try:
+    #     logdata = device.getlogdata()
+    #     if(logdata == None):
+    #         raise Exception("logdata is None")
+    #     print(color('SUCCESS: getting device logdata.', fore='green', style='bright'))
+    # except:
+    #     print(color('ERROR: getting device logdata.', fore='red', style='bright'))
 
-    try:
-        device.stop()
-        print(color('SUCCESS: stopping device before start.', fore='green', style='bright'))
-    except:
-        print(color('ERROR: stopping device before start.', fore='red', style='bright'))
+    # try:
+    #     device.stop()
+    #     print(color('SUCCESS: stopping device before start.', fore='green', style='bright'))
+    # except:
+    #     print(color('ERROR: stopping device before start.', fore='red', style='bright'))
 
-    try:
-        temp = device.getstate()
-        if(temp == False):
-            print(color('SUCCESS: getting device state (before start).', fore='green', style='bright'))
-        else:
-            raise Exception("device state is not stopped.")
-    except:
-        print(color('ERROR: getting device state (before start).', fore='red', style='bright'))
+    # try:
+    #     temp = device.getstate()
+    #     if(temp == False):
+    #         print(color('SUCCESS: getting device state (before start).', fore='green', style='bright'))
+    #     else:
+    #         raise Exception("device state is not stopped.")
+    # except:
+    #     print(color('ERROR: getting device state (before start).', fore='red', style='bright'))
 
-    try:
-        temp = device.getdevicetype()
-        if(temp == "ElwaSc20"):
-            print(color('SUCCESS: getting device type.', fore='green', style='bright'))
-        else:
-            raise Exception("device type does not match.")
-    except:
-        print(color('ERROR: getting device type.', fore='red', style='bright'))
+    # try:
+    #     temp = device.getdevicetype()
+    #     if(temp == "ElwaSc20"):
+    #         print(color('SUCCESS: getting device type.', fore='green', style='bright'))
+    #     else:
+    #         raise Exception("device type does not match.")
+    # except:
+    #     print(color('ERROR: getting device type.', fore='red', style='bright'))
 
-    try:
-        device.readallregisters()
-        print(color('SUCCESS: reading registers.', fore='green', style='bright'))
-    except:
-        print(color('ERROR: reading registers.', fore='red', style='bright'))
+    # try:
+    #     device.readallregisters()
+    #     print(color('SUCCESS: reading registers.', fore='green', style='bright'))
+    # except:
+    #     print(color('ERROR: reading registers.', fore='red', style='bright'))
 
     #DCS communication tests
     device = ElwaSc20(serialnr)
+    # device = ElwaSc20("1201061912110002")
     connection = DcsConnection(dcsserial, dcscryptokey, server, 50333)
     device.addserverconnection(connection)
     device.start()
@@ -423,7 +428,7 @@ if __name__ == "__main__":
         while True:
             print(color('[SerialDevice] DCS communication active. Press CTRL+C to stop', fore='blue', style='bright'))
             print(device)
-            device.showinfo()
+            print(device.getinfo())
             device.showcommunicationerrors()
             device.showcommunicationerrorsrate()
             logdata = device.getlogdata()
