@@ -15,7 +15,7 @@ sys.path.append(os.path.normpath(os.path.join(SCRIPT_DIR, PACKAGE_PARENT)))
 from mypvdevices.ServerConnection import ServerConnection
 from mypvdevices.Dataprocessor import Dataprocessor
 
-BUILDNR = "1.2112.11"
+BUILDNR = "1.2112.12"
 
 JOINTIMEOUTSECONDS = 5
 MONITORSLEEPTIMESECONDS = 10
@@ -101,7 +101,7 @@ class GeneralDevice:
         self._monitorthread.start()
         self._busCommunicationThread = threading.Thread(target=self._runbuscommunication, name="BusCommunication " + str(self._name))
         self._busCommunicationThread.start()
-        time.sleep(0.1)
+        time.sleep(10)
         for connection in self._serverconnections:
             connection.connect()
         logging.debug(str(self._name) + " started.")
@@ -241,16 +241,16 @@ class GeneralDevice:
                 logging.error(str(self._name) + ". BusCommunication Thread Error: " + str(e))
             
             executiontime = datetime.now() - buscommunicationstarttime
-            logging.debug(str(self._name) + " BusCommunication execution time: " + str(executiontime.total_seconds()))
+            logging.debug(str(self._name) + ". BusCommunication execution time: " + str(executiontime.total_seconds()))
             if(self._buscommunicationWaitTime > 0 and executiontime.total_seconds() > self._buscommunicationWaitTime):
-                logging.warning(str(self._name) + " BusCommunication execution time too high: " + str(executiontime.total_seconds()))
+                logging.warning(str(self._name) + ". BusCommunication execution time too high: " + str(executiontime.total_seconds()))
 
             time.sleep(random.random())        
             time.sleep(self._buscommunicationWaitTime)
 
     def _excecutebuscommunication(self):
         if self._buscommunicationWaitTime > 0:
-            logging.debug(str(self._name) + " BusCommunication running...")
+            logging.debug(str(self._name) + ". BusCommunication running...")
             try:
                 self.readallregisters()
             except Exception as e:
@@ -267,11 +267,12 @@ class GeneralDevice:
             try:
                 self._processchannels()
             except Exception as e:
-                logging.warning(str(self._name) + ". processing registers not successfull. " + str(e))
+                logging.warning(str(self._name) + ". Processing registers not successfull. " + str(e))
 
     def _syncsettings(self):
         errors = 0
         if self._setup != None:
+            setupchanged = False
             for name in self._setup:
                 channel = self._setup[name].channelname
                 if channel != None:
@@ -279,7 +280,7 @@ class GeneralDevice:
                         try:
                             channelvalue = self.getchannelvalue(channel)
                         except Exception as e:
-                            logging.warning(str(self._name) + " cannot get channelvalue")
+                            logging.warning(str(self._name) + ". Cannot get channelvalue")
                             channelvalue = None
                         setupvalue = self._setup[name].value
                         if channelvalue != setupvalue:
@@ -289,14 +290,16 @@ class GeneralDevice:
                                     self.setchannelvalue(channel, setupvalue)
                                 except Exception as e:
                                     logging.warning(str(self._name) + " cannot write channel while settings sync. Set " + str(name) + " to "+str(setupvalue)+". " + str(e))
-                                logging.info(str(self._name) + " forcing " + str(name) + " to "+str(setupvalue)+".")
+                                logging.info(str(self._name) + " forced " + str(name) + " to "+str(setupvalue)+".")
                             else:
                                 self._setup[name].value = channelvalue
-                                self.sendsetup()
+                                setupchanged = True
                                 logging.debug(str(self._name) + " changed setup. " + str(name) + "=" + str(channelvalue) + ".")
                     else:
                         logging.warning(str(self._name) + " channel for setting " + str(name) + " not found.")
                         errors += 1
+            if setupchanged:
+                self.sendsetup()
         if errors == 0:
             return True
         else:
@@ -371,10 +374,10 @@ class GeneralDevice:
                 registersToRead = element[1]
                 registertype = element[2]
             except Exception as e:
-                logging.error(str(self._name) + " invalid register definition: " + str(element))
+                logging.error(str(self._name) + ". Invalid register definition: " + str(element))
                 raise e
             if startAddress != None and registersToRead != None:
-                logging.debug(str(self._name) + " reading " + str(registersToRead) + " " + str(registertype) + " registers starting with " + str(startAddress))
+                logging.debug(str(self._name) + ". Reading " + str(registersToRead) + " " + str(registertype) + " registers starting with " + str(startAddress))
                 try:
                     registers = self.readregisters(startAddress, registersToRead, registertype) 
                 except Exception as e:
@@ -385,13 +388,13 @@ class GeneralDevice:
                 if registers == None or len(registers) == 0:
                     readerror += 1
             else:
-                logging.warning(str(self._name) + " register definition error: ", str(startAddress), str(registersToRead))
+                logging.warning(str(self._name) + ". Register definition error: ", str(startAddress), str(registersToRead))
         self._registers = allRegisters
         self._registerTimeStamp = time.time()
         if readerror == 0:
             self._registerLastSuccessfullReadTimeStamp = time.time()
         else:
-            logging.warning(str(self._name) + " reading register failed for " + str(readerror) + " blocks.")
+            logging.warning(str(self._name) + ". Reading register failed for " + str(readerror) + " blocks.")
 
     def readregister(self, registerId, registertype="holding"):
         if registerId == None:
@@ -508,10 +511,15 @@ class GeneralDevice:
 
         if self._datasetdefinitions != None:
             for dataset in self._datasetdefinitions:
+
+                if len(self._registers) == 0:
+                    # logging.debug(str(self._name) + " Cannot process channels. No data available")
+                    raise Exception("No data to process")
+
                 try:
                     value = self.getchannelvalue(dataset, False)
                 except Exception as e:
-                    logging.warning(str(self._name) + ", dataset=" + str(dataset) + ", channel not found")
+                    logging.debug(str(self._name) + ", dataset=" + str(dataset) + ", cannot get data from channel")
                     value = None
                 if value != None:
                     element = {
@@ -583,13 +591,12 @@ class GeneralDevice:
 
         try:
             logdata = self._logdata[datasetname]
+
+            if scale != None and ( isinstance(scale, int) or isinstance(scale, float)):
+                logdata = logdata * scale
+
             if casttype == "int":
-                value = int(logdata)
-
-                if scale != None and ( isinstance(scale, int) or isinstance(scale, float)):
-                    value = value * scale
-
-                return value
+                return int(logdata)
             else:
                 return logdata
         except Exception:
@@ -603,15 +610,17 @@ class GeneralDevice:
         self._logdataTimeStamp = None
         self._logdataStartTimeStamp = None
 
-    def showinfo(self):
+    def getinfo(self):
+        output = str(self._name)
         if self._datasetdefinitions != None:
             for dataset in self._datasetdefinitions:
                 try:
-                    logging.info(str(self._name) + " " + dataset + ": Live: " + str(self.getchannelvalue(dataset)) + ", Log: " + str(self.getlogvalue(dataset)) + ", HealthState: " + str(self.gethealthstate()))
+                    output += "\n" + dataset + ": Live: " + str(self.getchannelvalue(dataset)) + ", Log: " + str(self.getlogvalue(dataset))
                 except Exception as e:
-                    logging.debug(str(self._name) + " Error printing deviceInfo. " + str(e))
-        else:
-            logging.debug(str(self._name) + " no datasets defined")
+                    logging.debug(str(self._name) + " Error getting deviceInfo. " + str(e))
+
+        output += "\nHealthState: " + str(self.gethealthstate())
+        return output
 
     def showcommunicationerrors(self):
         logging.info(str(self._name) + " - Communication Errors: " +  str(self.getcommunicationerrorscounter()))
@@ -642,6 +651,10 @@ class GeneralDevice:
                 logging.info("No channeldata available. Registers have not been read yet. Run read registers first. " + str(self._name) + ", " + str(channel))
                 raise Exception("No register data available. Register not read?")
 
+            if len(self._registers) == 0:
+                logging.info("No channeldata available. Data is empty. " + str(self._name) + ", " + str(channel))
+                raise Exception("No register data available.")
+
             if self._registerTimeStamp != None:
                 difference = time.time() - self._registerTimeStamp
                 if difference > self._datamaximumage:
@@ -662,7 +675,7 @@ class GeneralDevice:
                 try:
                     registerValues.append(self._registers[register])
                 except Exception as e:
-                    logging.warning("Register " + str(e) + " for channel " + str(channel) + " not found. Device: " + str(self._name))
+                    logging.debug("Register " + str(e) + " for channel " + str(channel) + " not found. Device: " + str(self._name))
                     raise Exception("Register not found: " + str(e))
 
             if config.channeltype == None:
@@ -1192,7 +1205,7 @@ class GeneralDevice:
 #     try:
 #         while True:
 #             print(color('[GeneralDevice] DCS communication active. Press CTRL+C to stop', fore='blue', style='bright'))
-#             device.showinfo()
+#             print(device.getinfo())
 #             device.showcommunicationerrors()
 #             device.showcommunicationerrorsrate()
 #             time.sleep(10)
